/**
 * @file
 * VuoList_VuoData C type definition.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoData_H
#define VuoList_VuoData_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoData VuoList_VuoData
 * A list of @ref VuoData elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoData_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoData elements.
 */
typedef const struct VuoList_VuoData_struct { void *l; } * VuoList_VuoData;
/// @}
#endif

/**
 * Creates a new list of @ref VuoData elements.
 */
VuoList_VuoData VuoListCreate_VuoData(void);

/**
 * Creates a new list of `count` instances of `value`.
 *
 * Use this in conjunction with @ref VuoListGetData_VuoData to quickly initialize a large list.
 */
VuoList_VuoData VuoListCreateWithCount_VuoData(const unsigned long count, const VuoData value);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoData VuoListCopy_VuoData(const VuoList_VuoData list);

/**
 * Returns the @ref VuoData at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoData VuoListGetValue_VuoData(const VuoList_VuoData list, const unsigned long index);

/**
 * Returns a pointer to a C array containing the list items.
 *
 * Use this if you need a fast way to get or change multiple list items.
 *
 * You can modify values in the list by changing them in this array.
 * Just don't attempt to access beyond the list size.
 *
 * The pointer becomes invalid if you modify the list size using other functions
 * (e.g., Insert, Prepend, Append, Cut, Remove);
 * if you use those functions, just get a new pointer by calling this function again.
 *
 * If the list has no items, returns NULL.
 *
 * The pointer remains owned by the list; don't free it.
 */
VuoData *VuoListGetData_VuoData(const VuoList_VuoData list);

/**
 * Changes the @ref VuoData at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoData(const VuoList_VuoData list, const VuoData value, const unsigned long index);

/**
 * Inserts the `VuoData` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoData(const VuoList_VuoData list, const VuoData value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoData(VuoList_VuoData list, const VuoData value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoData(VuoList_VuoData list, const VuoData value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoData(VuoList_VuoData list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoData(VuoList_VuoData list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoData(VuoList_VuoData list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoData(VuoList_VuoData list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoData(VuoList_VuoData list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoData(VuoList_VuoData list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoData(VuoList_VuoData list);

/**
 * Removes the `VuoData` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoData(VuoList_VuoData list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoData(const VuoList_VuoData list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoData VuoList_VuoData_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoData_getJson(const VuoList_VuoData value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoData_getSummary(const VuoList_VuoData value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoData VuoList_VuoData_makeFromString(const char *str);
char * VuoList_VuoData_getString(const VuoList_VuoData value);
/// @}

/**
 * @}
 */

#endif
