/**
 * @file
 * Logging functions.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VUOLOG_H
#define VUOLOG_H

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <unistd.h>
#include <asl.h>
#include <math.h>

/**
 * @ingroup DevelopingNodeClasses DevelopingTypes DevelopingLibraryModules
 * @defgroup VuoModuleDebug Module Debugging
 * Macros to help with debugging.
 *
 * @{
 */


/// @cond
/**
 * Returns the number of seconds (including fractional seconds) since midnight 1970.01.01 GMT.
 */
static double VuoLogGetTime(void)
{
	struct timeval t;
	gettimeofday(&t, NULL);
	return t.tv_sec + t.tv_usec / 1000000.;
}

static double VuoLogStartTime;	///< The time when this module was loaded.

/**
 * Stores the time at which this module was loaded, for use by @ref VuoLogGetElapsedTime().
 */
static void __attribute__((constructor)) VuoLogInit(void)
{
	VuoLogStartTime = VuoLogGetTime();
}

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused"
/**
 * Returns the number of seconds (including fractional seconds) since this module was loaded.
 */
static double VuoLogGetElapsedTime()
{
	return VuoLogGetTime() - VuoLogStartTime;
}

#ifndef DOXYGEN
	#define VUO_FORMAT_ATTRIBUTE(formatStringIndex, formatArgumentsIndex) __attribute__((format(printf, formatStringIndex, formatArgumentsIndex)))
#endif
static void VuoLog(const char *file, const unsigned int line, const char *function, const char *format, ...) VUO_FORMAT_ATTRIBUTE(4,5);

/**
 * Outputs a message to the system log and to `stderr`.
 */
static void VuoLog(const char *file, const unsigned int line, const char *function, const char *format, ...)
{
	va_list args;

	va_start(args, format);
	int size = vsnprintf(NULL, 0, format, args);
	va_end(args);

	char *formattedString = (char *)malloc(size+1);
	va_start(args, format);
	vsnprintf(formattedString, size+1, format, args);
	va_end(args);

	char *formattedFunction = NULL;

	// This may be a mangled function name of the form `__6+[f g]_block_invoke`.
	// Trim the prefix and suffix, since the line number is sufficient to locate the code within the function+block.
	if (function[0] == '_' && function[1] == '_')
	{
		int actualFunctionLength = atoi(function + 2);
		if (actualFunctionLength)
			formattedFunction = strndup(function + 3 + (int)log10(actualFunctionLength), actualFunctionLength);
		else
			formattedFunction = strndup(function + 2, strchr(function + 2, '_') - (function + 2));
	}

	// Add a trailing `()`, unless it's an Objective-C method.
	{
		const char *f = formattedFunction ? formattedFunction : function;
		if (f[strlen(f) - 1] != ']')
		{
			char *f2 = (char *)malloc(strlen(f) + 3);
			strcpy(f2, f);
			strcat(f2, "()");
			if (formattedFunction)
				free(formattedFunction);
			formattedFunction = f2;
		}
	}

	double time = VuoLogGetElapsedTime();

	fprintf(stderr, "\033[38;5;%dm# pid=%d  t=%8.4fs %27s:%-4d  %41s \t%s\033[0m\n", getpid()%212+19, getpid(), time, file, line, formattedFunction ? formattedFunction : function, formattedString);

	aslmsg msg = asl_new(ASL_TYPE_MSG);
	asl_set(msg, ASL_KEY_READ_UID, "-1");
	asl_log(NULL, msg, ASL_LEVEL_WARNING, "%s:%d  %s()  %s", file, line, function, formattedString);
	asl_free(msg);

	free(formattedString);
}
#pragma clang diagnostic pop
/// @endcond

/**
 * Prints the name of the file and function to the system log and to `stderr` (and implicitly flushes the output buffer).
 *
 * \eg{
 * void nodeEvent()
 * {
 *     VL();
 * }
 * }
 *
 * @hideinitializer
 */
#define VL() VuoLog(__FILE__, __LINE__, __func__, "")

/**
 * Prints the name of the file and function, and `printf()`-style format/arguments, to the system log and to `stderr` (and implicitly flushes the output buffer).
 *
 * Use this for debugging only; don't commit files with `VLog` calls.  To log messages intended for end users, see @ref VUserLog.
 *
 * \eg{
 * void nodeEvent(VuoInputData(VuoInteger, "42") number)
 * {
 *     VLog("%d", number);
 * }
 * }
 *
 * @hideinitializer
 */
#define VLog(format, ...) VuoLog(__FILE__, __LINE__, __func__, format, ##__VA_ARGS__)

/**
 * Same as @ref VLog, but for messages intended for end users.
 *
 * @hideinitializer
 */
#define VUserLog(format, ...) VuoLog(__FILE__, __LINE__, __func__, format, ##__VA_ARGS__)


#ifndef __ASSERT_MACROS_DEFINE_VERSIONS_WITHOUT_UNDERSCORES
	/// Avoid conflict between Cocoa and LLVM headers.
	#define __ASSERT_MACROS_DEFINE_VERSIONS_WITHOUT_UNDERSCORES 0
#endif
#include <CoreFoundation/CoreFoundation.h>

#include <AvailabilityMacros.h>
#if (MAC_OS_X_VERSION_MIN_REQUIRED == MAC_OS_X_VERSION_10_6) || (MAC_OS_X_VERSION_MIN_REQUIRED == MAC_OS_X_VERSION_10_7)
	#include <ApplicationServices/ApplicationServices.h>
#else
	#include <CoreGraphics/CoreGraphics.h>
	#include <CoreText/CoreText.h>
#endif

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused"
	/**
	 * Returns true if debug mode is enabled.
	 *
	 * Users can enable debug mode by executing `defaults write org.vuo.Editor debug -boolean true`,
	 * and can disable it by executing `defaults delete org.vuo.Editor debug`.
	 *
	 * Nodes should use this to control verbose logging.
	 */
	static bool VuoIsDebugEnabled(void)
	{
		static dispatch_once_t checked;
		static bool debug = false;
		dispatch_once(&checked, ^{
						  debug = CFPreferencesGetAppBooleanValue(CFSTR("debug"), CFSTR("org.vuo.Editor"), NULL);
					  });
		return debug;
	}

	/**
	 * Returns a C string description of `variable`, a reference to a CoreFoundation object.
	 *
	 * The caller is responsible for `free()`ing the returned string.
	 */
	static char *VuoLog_copyCFDescription(CFTypeRef variable)
	{
		if (!variable)
			return strdup("(null)");

		CFStringRef d = CFCopyDescription(variable);
		CFIndex len = CFStringGetLength(d)+1;
		char *z = (char *)malloc(len);
		CFStringGetCString(d, z, len, kCFStringEncodingUTF8);
		CFRelease(d);
		return z;
	}
#pragma clang diagnostic pop


/**
 * Prints the name of the current file and function, and the address and description of the specified `heapPointer`, to the system log and to `stderr` (and implicitly flushes the output buffer).
 *
 * \eg{
 * void nodeEvent(VuoInputData(VuoShader) shader)
 * {
 *     VLogHeap(shader);
 * }
 * }
 *
 * @hideinitializer
 */
#define VLogHeap(heapPointer) do { \
	char *description = VuoHeap_getDescription(heapPointer); \
	VLog("%s = %p (registered at %s)", #heapPointer, heapPointer, description); \
	free(description); \
	} while(0)

/**
 * @}
 */

#endif // VUOLOG_H
