/**
 * @file
 * VuoVideoDecoder interface.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */
#pragma once

#ifdef __cplusplus
extern "C"
{
#include "module.h"
#include "VuoVideoFrame.h"
#include "VuoAudioSamples.h"
#include "VuoList_VuoAudioSamples.h"
#include "VuoUrl.h"
#endif

/**
 * A virtual class that defines methods for video decoders to implement.  Allows for Vuo to
 * choose which video playback framework is most appropriate for the codec.
 */
class VuoVideoDecoder
{
protected:
	/// VuoVideoDecoder constructor and inheriting class constructors should remain internal, allowing Create() to 
	/// allocate and initialize new decoders.  This is because initialization could potentially fail (bad path, incompatible
	/// codec, etc).
	VuoVideoDecoder(VuoUrl url)
	{
		mPlaybackRate = 1.;
	 	mVideoPath = VuoUrl_normalize(url, false);
		VuoRetain(mVideoPath);
	}

	/// How fast and in which direction the video is being decoded. Negative values are allowed.  A value of 1 means
	/// video is playing at the speed requested by the file, where 2 is twice as fast, and -1 is reversed.
	double mPlaybackRate;

public:

	/// Destructor!
	virtual ~VuoVideoDecoder()
	{
		if(mVideoPath != NULL)
			VuoRelease(mVideoPath);
	}

	/// The path to the movie file.
	VuoUrl mVideoPath;

	/// Returns the next video frame in sequence (forwards or backwards depending on mPlaybackRate).
	virtual bool NextVideoFrame(VuoVideoFrame* frame) = 0;

	/// Return the next list of audio samples.  The timestamp for the current sample is put in `timestamp`.
	virtual bool NextAudioFrame(VuoList_VuoAudioSamples audio, double* timestamp) = 0;

	/// Seek the playhead to `second`.
	virtual bool SeekToSecond(double second) = 0;

	/// Return true if the file contains an audio track.
	virtual bool ContainsAudio() = 0;

	/// The timestamp of the first frame in this video (in seconds).
	virtual double StartTimestamp() = 0;

	/// The timestamp of the last frame in this video (in seconds).
	virtual double EndTimestamp() = 0;

	/// The duration of this video in seconds
	virtual double GetDuration() = 0;

	/// The timestamp of the last accessed frame (in seconds).
	virtual double GetLastDecodedVideoTimeStamp() = 0;

	/// Set the playback speed.
	virtual void SetPlaybackRate(double rate) = 0;

	/// Return the file path to the video.
	const char* GetPath() { return (const char*) mVideoPath; }
};

#ifdef __cplusplus
}
#endif
